#!/usr/bin/perl

# exim_perf.pl - gathers performance data from the exim log file

use strict;
use File::stat qw(:FIELDS);

my $exim_log = '/var/log/exim/main.log';
my $logchunk = get_newest($exim_log);

open STATS, "|eximstats > $exim_log.temp" or die;

foreach my $line (@$logchunk) {
        print STATS $line;
}

close STATS;

my $clamcheck = 0;
my $local_delivery = 0;
my $remote_smtp = 0;
my $spamcheck = 0;
my $trend_micro = 0;
my $remote_smtp_single_recipient_domains = 0;

open STATS, "<$exim_log.temp" or die;

while (my $line = <STATS>) {
        chomp $line;
        if ($line =~ m/clamcheck .* ([0-9]+)$/) {
                $clamcheck = $1;
        }
        if ($line =~ m/local_delivery .* ([0-9]+)$/) {
                $local_delivery = $1;
        }
        if ($line =~ m/remote_smtp .* ([0-9]+)$/) {
                $remote_smtp = $1;
        }
        if ($line =~ m/spamcheck .* ([0-9]+)$/) {
                $spamcheck = $1;
        }
        if ($line =~ m/trend_micro .* ([0-9]+)$/) {
                $trend_micro = $1;
        }
        if ($line =~ m/remote_smtp_single_recipient_domains .* ([0-9]+)$/) {
                $remote_smtp_single_recipient_domains = $1;
        }
}
close STATS;

print "clamcheck\n";
print "$clamcheck\n";
print "local_delivery\n";
print "$local_delivery\n";
print "remote_smtp\n";
print "$remote_smtp\n";
print "spamcheck\n";
print "$spamcheck\n";
print "trend_micro\n";
print "$trend_micro\n";
print "remote_smtp_single_recipient_domains\n";
print "$remote_smtp_single_recipient_domains\n";

exit (0);

sub get_newest ($) {
        my $file = shift;
        my $position= 0;
        my $timestamp = 1000000000000000;
        my @log;

        if (-e "$exim_log.state") {
                open FILE, "<$exim_log.state" or die;
                my $line = <FILE>;
                chomp $line;
                ($position, $timestamp) = split " ", $line;
                close FILE;
        }

        open FILE, "<$exim_log" or die;
        my $st = stat("$exim_log.1") or die;

        if ($st->ctime > $timestamp) {
                # File has been rotated, read the old file first.
                open OLDFILE, "<$exim_log.1" or die;
                seek (OLDFILE, $position, 0);
                while (my $line = <OLDFILE>) {
                        push @log, $line;
                }
                close OLDFILE;
                $position = 0;
        }

        seek (FILE, $position, 0);

        # now read the new file.
        while (my $line = <FILE>) {
                push @log, $line;
        }
        $position = tell(FILE);

        close FILE;

        open FILE, ">$exim_log.state" or die;
        print FILE "$position " . $st->ctime . "\n";
        close FILE;

        return \@log;
}
